@@ -14,7 +14,7 @@ REQDN=reqDN
 STRMASK=default
 export REQDN STRMASK
 
-openssl version
+openssl version 1>&2
 
 set -ex
 
@@ -75,21 +75,21 @@ ${REQ} -new -key ${srcdir}/server.key -out altname4.csr
 csr_fields "Good ipAddress altname Dept" nowhere.example.com | \
 ${REQ} -new -key ${srcdir}/server.key -out altname5.csr
 
-csr_fields "Bad ipAddress altname Dept" nowhere.example.com | \
+csr_fields "Bad ipAddress altname 1 Dept" nowhere.example.com | \
 ${REQ} -new -key ${srcdir}/server.key -out altname6.csr
 
-csr_fields "Bad ipAddress altname Dept" nowhere.example.com | \
+csr_fields "Bad ipAddress altname 2 Dept" nowhere.example.com | \
 ${REQ} -new -key ${srcdir}/server.key -out altname7.csr
 
-csr_fields "Bad ipAddress altname Dept" nowhere.example.com | \
+csr_fields "Bad ipAddress altname 3 Dept" nowhere.example.com | \
 ${REQ} -new -key ${srcdir}/server.key -out altname8.csr
 
+csr_fields "Bad Hostname Department" nohost.example.com | \
+${REQ} -new -key ${srcdir}/server.key -out wrongcn.csr
+
 csr_fields "Self-Signed" | \
 ${MKCERT} -key ${srcdir}/server.key -out ssigned.pem
 
-csr_fields "Bad Hostname Department" nohost.example.com | \
-${MKCERT} -key ${srcdir}/server.key -out wrongcn.pem
-
 # default => T61String
 csr_fields "`echo -e 'H\0350llo World'`" localhost |
 ${MKCERT} -key ${srcdir}/server.key -out t61subj.cert
@@ -148,7 +148,8 @@ First OU Dept" | ${REQ} -new -key ${srcdir}/server.key -out twoou.csr
 
 ### don't put ${REQ} invocations after here
 
-for f in server client twocn caseless cnfirst missingcn justmail twoou wildcard; do
+for f in server client twocn caseless cnfirst \
+    missingcn justmail twoou wildcard wrongcn; do
   ${CA} -days 900 -in ${f}.csr -out ${f}.cert
 done
 
